import React, { useContext, useRef, useState, useEffect, useMemo, useCallback } from 'react';

/**
 * The map context
 */
const GoogleMapContext = React.createContext({
  loading: true
});

/**
 * Hook to get global map instance
 */
const useGoogleMap = () => useContext(GoogleMapContext);

/**
 * Hook to get a Google Maps Places Autocomplete instance
 * monitoring an input field
 */
const useAutocomplete = props => {
  const {
    inputField,
    options,
    onPlaceChanged
  } = props;
  const placeChangedHandler = useRef(onPlaceChanged);
  const {
    map
  } = useGoogleMap();
  const [autocomplete, setAutocomplete] = useState(null);
  // Initializes the Google Maps Places Autocomplete
  useEffect(() => {
    // Wait for map and input element to be initialized
    if (!map || !inputField) {
      return () => {};
    }
    if (!google.maps.places) {
      throw Error("Autocomplete library missing. Add 'places' to the libraries array of GoogleMapProvider.");
    }
    // Create Autocomplete instance
    const autocompleteInstance = new google.maps.places.Autocomplete(inputField, options);
    setAutocomplete(autocompleteInstance);
    // Add places change listener to Autocomplete
    autocompleteInstance.addListener('place_changed', () => {
      const place = autocompleteInstance.getPlace();
      placeChangedHandler.current && placeChangedHandler.current(place);
    });
    // Clear listeners on unmount
    return () => {
      autocompleteInstance && google.maps.event.clearInstanceListeners(autocompleteInstance);
    };
  }, [map, inputField, options]);
  return autocomplete;
};

/**
 * Hook to get Google Maps Places Directions Service instance
 */
const useDirections = (props = {}) => {
  const {
    renderOnMap,
    renderOptions
  } = props;
  const {
    map,
    loading
  } = useGoogleMap();
  // Creates a Directions Service instance
  const directionsService = useMemo(() => {
    // Wait for map to be initialized
    if (!map || loading) {
      return null;
    }
    return new google.maps.DirectionsService();
  }, [map, loading]);
  // Creates a Directions Renderer instance
  const directionsRenderer = useMemo(() => {
    // Wait for map to be initialized
    if (!map || !renderOnMap) {
      return null;
    }
    const renderer = new google.maps.DirectionsRenderer(renderOptions);
    renderer.setMap(map);
    return renderer;
  }, [map, renderOnMap]);
  // Updates the directions renderer options
  useEffect(() => {
    if (!directionsRenderer) {
      return;
    }
    directionsRenderer.setOptions(renderOptions || {});
  }, [renderOptions]);
  // Custom Directions route request
  const findRoute = useCallback(request => new Promise((resolve, reject) => {
    if (directionsService) {
      directionsService.route(request, (result, status) => {
        if (status === google.maps.DirectionsStatus.OK) {
          resolve(result);
        } else {
          reject(status);
        }
      });
    }
  }), [directionsService]);
  // Custom Directions route request followed by directions rendering
  const findAndRenderRoute = useCallback(request => new Promise((resolve, reject) => {
    if (directionsService) {
      directionsService.route(request, (result, status) => {
        if (status === google.maps.DirectionsStatus.OK) {
          if (directionsRenderer) {
            directionsRenderer.setDirections(result);
          }
          resolve(result);
        } else {
          reject(status);
        }
      });
    }
  }), [directionsService, directionsRenderer]);
  // Renders directions route of given index
  const renderRouteOfIndex = index => {
    if (directionsRenderer) {
      directionsRenderer.setRouteIndex(index);
    }
  };
  return {
    directionsService,
    findRoute: directionsService && findRoute,
    findAndRenderRoute: directionsService && directionsRenderer && findAndRenderRoute,
    renderRouteOfIndex
  };
};

/**
 * Hook to get Distance Matrix Service instance
 */
const useDistanceMatrix = () => {
  const {
    map
  } = useGoogleMap();
  // Creates a Distance Matrix Service instance
  const distanceMatrixService = useMemo(() => {
    // Wait for map to be initialized
    if (!map) {
      return null;
    }
    if (!google.maps.DistanceMatrixService) {
      throw Error('Distance Matrix library missing.');
    }
    return new google.maps.DistanceMatrixService();
  }, [map]);
  return distanceMatrixService;
};

/**
 * Hook to get Elevation Service instance
 */
const useElevationService = () => {
  const {
    map
  } = useGoogleMap();
  // Creates an Elevation Service instance
  const elevationService = useMemo(() => {
    // Wait for map to be initialized
    if (!map) {
      return null;
    }
    return new google.maps.ElevationService();
  }, [map]);
  return elevationService;
};

/**
 * Hook to get Google Maps Geocoder instance
 */
const useGeocoder = () => {
  const {
    map
  } = useGoogleMap();
  // Creates a Geocoder instance
  const geocoder = useMemo(() => {
    // Wait for Google Maps API to be initialized
    if (!map) {
      return null;
    }
    return new google.maps.Geocoder();
  }, [map]);
  return geocoder;
};

/**
 * Hook to get Max Zoom Service instance
 */
const useMaxZoomService = () => {
  const {
    map
  } = useGoogleMap();
  // Creates a Max Zoom Service instance
  const maxZoomService = useMemo(() => {
    // Wait for map to be initialized
    if (!map) {
      return null;
    }
    return new google.maps.MaxZoomService();
  }, [map]);
  return maxZoomService;
};

/**
 * Hook to get Google Maps Places Service instance
 */
const usePlacesService = () => {
  const {
    map
  } = useGoogleMap();
  // Creates a Places Service instance
  const placesService = useMemo(() => {
    // Wait for map to be initialized
    if (!map) {
      return null;
    }
    if (!google.maps.places) {
      throw Error("Places library missing. Add 'places' to the libraries array of GoogleMapProvider.");
    }
    return new google.maps.places.PlacesService(map);
  }, [map]);
  return placesService;
};

export { useAutocomplete, useDirections, useDistanceMatrix, useElevationService, useGeocoder, useGoogleMap, useMaxZoomService, usePlacesService };
